import pygame
import sys
from enum import Enum, auto
from src.snake import Snake, Direction
from src.food import Food
from src.menu import Menu, GameMode

class GameState(Enum):
    MENU = auto()
    PLAYING = auto()
    GAME_OVER = auto()
    PAUSED = auto()

class GameRules:
    def __init__(self):
        self.wrap_around = False  # Whether snake wraps around screen edges
        self.speed_increase = True  # Whether snake speeds up as it grows
        self.min_move_cooldown = 50  # Minimum movement delay in milliseconds
        self.initial_move_cooldown = 100  # Initial movement delay

class Game:
    def __init__(self):
        # Initialize pygame and font system
        if not pygame.get_init():
            pygame.init()
        if not pygame.font.get_init():
            pygame.font.init()
        
        # Initialize display
        self.width = 800
        self.height = 600
        self.screen = pygame.display.set_mode((self.width, self.height))
        pygame.display.set_caption("AI Snake Game")
        
        # Initialize clock
        self.clock = pygame.time.Clock()
        self.fps = 60
        
        # Game rules
        self.rules = GameRules()
        
        # Game objects
        self.block_size = 20
        self.menu = Menu(self.width, self.height)
        self.reset_game()
        
        # Game state
        self.state = GameState.MENU  # Start in menu state
        self.running = True
        self.game_mode = None
        
        # Debug mode
        self.debug = False

    def reset_game(self):
        """Reset the game state for a new game"""
        self.snake = Snake((self.width // 2, self.height // 2), self.block_size)
        self.food = Food(self.block_size)
        self.food.spawn(self.width, self.height, self.snake.body)
        self.score = 0

    def handle_events(self):
        current_time = pygame.time.get_ticks()
        
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                self.running = False
            elif event.type == pygame.KEYDOWN:
                if event.key == pygame.K_ESCAPE:
                    if self.state == GameState.PLAYING:
                        self.state = GameState.PAUSED
                    elif self.state == GameState.PAUSED:
                        self.state = GameState.PLAYING
                    elif self.state == GameState.MENU:
                        self.running = False
                elif event.key == pygame.K_F3:  # Toggle debug mode
                    self.debug = not self.debug
                elif event.key == pygame.K_w:  # Toggle wrap-around mode in debug
                    if self.debug:
                        self.rules.wrap_around = not self.rules.wrap_around
                
                if self.state == GameState.MENU:
                    # Handle menu input
                    selected_mode = self.menu.handle_input(event)
                    if selected_mode is not None:
                        if selected_mode == GameMode.PLAYER:
                            self.game_mode = GameMode.PLAYER
                            self.state = GameState.PLAYING
                        elif selected_mode in (GameMode.AI_EASY, GameMode.AI_MEDIUM, GameMode.AI_HARD):
                            self.game_mode = selected_mode
                            self.state = GameState.PLAYING
                        else:  # Quit selected
                            self.running = False
                
                elif self.state == GameState.PLAYING:
                    # Handle snake direction
                    if event.key == pygame.K_UP:
                        self.snake.change_direction(Direction.UP, current_time)
                    elif event.key == pygame.K_DOWN:
                        self.snake.change_direction(Direction.DOWN, current_time)
                    elif event.key == pygame.K_LEFT:
                        self.snake.change_direction(Direction.LEFT, current_time)
                    elif event.key == pygame.K_RIGHT:
                        self.snake.change_direction(Direction.RIGHT, current_time)
                
                elif self.state == GameState.GAME_OVER:
                    # Any key to return to menu in game over state
                    self.state = GameState.MENU
                    self.reset_game()

    def update(self):
        if self.state == GameState.PLAYING:
            # Move snake
            if self.snake.move(pygame.time.get_ticks()):
                # Handle wrap-around
                if self.rules.wrap_around:
                    self.snake.wrap_position(self.width, self.height)
                
                # Check collisions
                if self.snake.check_collision(self.width, self.height, self.rules.wrap_around):
                    self.state = GameState.GAME_OVER
                    return
                
                # Check food collision
                if self.food.check_collision(self.snake.body[0]):
                    self.snake.grow()
                    self.score += 1
                    self.food.spawn(self.width, self.height, self.snake.body)
                    
                    # Increase speed if enabled
                    if self.rules.speed_increase:
                        self.snake.move_cooldown = max(
                            self.rules.min_move_cooldown,
                            self.rules.initial_move_cooldown - self.score * 2
                        )

    def draw_grid(self):
        """Draw the game grid (debug mode)"""
        for x in range(0, self.width, self.block_size):
            pygame.draw.line(self.screen, (50, 50, 50), (x, 0), (x, self.height))
        for y in range(0, self.height, self.block_size):
            pygame.draw.line(self.screen, (50, 50, 50), (0, y), (self.width, y))

    def draw_debug_info(self):
        """Draw debug information"""
        font = pygame.font.Font(None, 24)
        debug_info = [
            f"FPS: {int(self.clock.get_fps())}",
            f"Snake Length: {len(self.snake.body)}",
            f"Snake Head: {self.snake.body[0]}",
            f"Food Pos: {self.food.position}",
            f"Game State: {self.state.name}",
            f"Game Mode: {self.game_mode.name if self.game_mode else 'None'}",
            f"Wrap Around: {self.rules.wrap_around}",
            f"Move Cooldown: {self.snake.move_cooldown}ms"
        ]
        
        for i, text in enumerate(debug_info):
            surface = font.render(text, True, (0, 255, 0))
            self.screen.blit(surface, (10, self.height - 200 + i * 25))

    def render(self):
        # Clear screen
        self.screen.fill((0, 0, 0))  # Black background
        
        # Draw grid in debug mode
        if self.debug and self.state != GameState.MENU:
            self.draw_grid()
        
        if self.state == GameState.MENU:
            self.menu.draw(self.screen)
        
        elif self.state == GameState.PLAYING or self.state == GameState.PAUSED:
            # Draw game objects
            self.snake.draw(self.screen)
            self.food.draw(self.screen)
            
            # Draw score
            font = pygame.font.Font(None, 36)
            score_text = font.render(f'Score: {self.score}', True, (255, 255, 255))
            self.screen.blit(score_text, (10, 10))
            
            # Draw game mode
            mode_text = font.render(f'Mode: {self.game_mode.name}', True, (255, 255, 255))
            self.screen.blit(mode_text, (10, 50))
            
            # Draw pause indicator
            if self.state == GameState.PAUSED:
                pause_text = font.render('PAUSED', True, (255, 255, 255))
                text_rect = pause_text.get_rect(center=(self.width//2, self.height//2))
                self.screen.blit(pause_text, text_rect)
        
        elif self.state == GameState.GAME_OVER:
            font = pygame.font.Font(None, 74)
            text = font.render('Game Over!', True, (255, 0, 0))
            score_text = font.render(f'Score: {self.score}', True, (255, 255, 255))
            restart_text = font.render('Press any key for menu', True, (255, 255, 255))
            
            text_rect = text.get_rect(center=(self.width//2, self.height//2 - 50))
            score_rect = score_text.get_rect(center=(self.width//2, self.height//2 + 50))
            restart_rect = restart_text.get_rect(center=(self.width//2, self.height//2 + 150))
            
            self.screen.blit(text, text_rect)
            self.screen.blit(score_text, score_rect)
            self.screen.blit(restart_text, restart_rect)
        
        # Draw debug info
        if self.debug:
            self.draw_debug_info()
        
        # Update display
        pygame.display.flip()

    def run(self):
        while self.running:
            self.handle_events()
            self.update()
            self.render()
            self.clock.tick(self.fps) 