import pygame
from enum import Enum, auto
from typing import List, Tuple

class Direction(Enum):
    UP = auto()
    DOWN = auto()
    LEFT = auto()
    RIGHT = auto()

class Snake:
    def __init__(self, start_pos: Tuple[int, int], block_size: int):
        self.block_size = block_size
        self.direction = Direction.RIGHT
        self.body = [start_pos]  # Head is at index 0
        self.growing = False
        
        # Movement cooldown
        self.move_cooldown = 100  # milliseconds
        self.last_move_time = 0
        
        # Direction change cooldown
        self.direction_change_cooldown = 50  # milliseconds
        self.last_direction_change = 0
        self.queued_direction = None

    def move(self, current_time: int) -> bool:
        """Move the snake if enough time has passed. Returns True if moved."""
        if current_time - self.last_move_time < self.move_cooldown:
            return False

        # Apply queued direction change if it exists and is valid
        if self.queued_direction and current_time - self.last_direction_change >= self.direction_change_cooldown:
            self._apply_direction_change(self.queued_direction)
            self.queued_direction = None

        # Update position
        head = self.body[0]
        new_head = self._get_new_head_position(head)
        
        # Insert new head
        self.body.insert(0, new_head)
        
        # Remove tail if not growing
        if not self.growing:
            self.body.pop()
        else:
            self.growing = False
        
        self.last_move_time = current_time
        return True

    def _get_new_head_position(self, head: Tuple[int, int]) -> Tuple[int, int]:
        x, y = head
        if self.direction == Direction.UP:
            return (x, y - self.block_size)
        elif self.direction == Direction.DOWN:
            return (x, y + self.block_size)
        elif self.direction == Direction.LEFT:
            return (x - self.block_size, y)
        else:  # Direction.RIGHT
            return (x + self.block_size, y)

    def grow(self):
        """Mark the snake to grow on next move"""
        self.growing = True

    def check_collision(self, width: int, height: int, wrap_around: bool = False) -> bool:
        """
        Check if snake has collided with walls or itself.
        
        Args:
            width: Game area width
            height: Game area height
            wrap_around: If True, snake wraps around screen edges instead of colliding
        """
        head = self.body[0]
        
        if not wrap_around:
            # Check wall collision
            if (head[0] < 0 or head[0] >= width or 
                head[1] < 0 or head[1] >= height):
                return True
        
        # Check self collision (skip head)
        if head in self.body[1:]:
            return True
        
        return False

    def wrap_position(self, width: int, height: int):
        """Wrap snake's head position around screen edges"""
        head_x, head_y = self.body[0]
        wrapped_head = (
            head_x % width if head_x != width else 0,
            head_y % height if head_y != height else 0
        )
        self.body[0] = wrapped_head

    def _apply_direction_change(self, new_direction: Direction) -> bool:
        """Internal method to actually change direction"""
        opposites = {
            Direction.UP: Direction.DOWN,
            Direction.DOWN: Direction.UP,
            Direction.LEFT: Direction.RIGHT,
            Direction.RIGHT: Direction.LEFT
        }
        
        if opposites[new_direction] != self.direction:
            self.direction = new_direction
            return True
        return False

    def change_direction(self, new_direction: Direction, current_time: int):
        """
        Queue a direction change if it's valid and cooldown has passed.
        
        Args:
            new_direction: The desired new direction
            current_time: Current game time in milliseconds
        """
        if current_time - self.last_direction_change < self.direction_change_cooldown:
            # Queue the direction change for next movement
            self.queued_direction = new_direction
            return
            
        if self._apply_direction_change(new_direction):
            self.last_direction_change = current_time

    def draw(self, screen: pygame.Surface):
        """Draw the snake on the screen"""
        # Draw head in a slightly different color
        head_color = (0, 200, 0)  # Darker green for head
        body_color = (0, 255, 0)  # Regular green for body
        
        # Draw body segments
        for i, segment in enumerate(self.body):
            color = head_color if i == 0 else body_color
            pygame.draw.rect(
                screen,
                color,
                pygame.Rect(segment[0], segment[1], self.block_size, self.block_size)
            ) 